/* (c) Peter Mueller, 2014 - All rights reserved. Reproduction, modification,
   use or disclosure to third parties without express
   authority is forbidden.
 */

#include <msp430.h>
#include <intrinsics.h>
#include "adc10.h"
#include "ringbuf.h"
#include "../generated/toggle_ext.h"
#include "../generated/toggle.h"

extern FIFO_T buf1;

static uint16_t readADC10(void);


// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{ 
    // enqueue event
    uint16_t adcVal = readADC10()*32;           //Ubatt=100%=1024

	__disable_interrupt();
    if(adcVal>15000){
      fifoPut(&buf1, evPowerOk);
    }else{
      fifoPut(&buf1, evPowerFail);
    }
	__enable_interrupt();
    
    ADC10CTL0 &= ~(ENC + ADC10SC);              // stop after one conversion
    __bic_SR_register_on_exit(LPM3_bits);
}


// Setup ADC10 for channel 0 and start single conversion
void initADC10(){
  
  // initialize ADC  
  ADC10CTL1 = ADC10DIV_3 + INCH_0;        // select Analog Input Pin P2.0
  ADC10CTL0 = SREF_0 | ADC10SHT_3 | ADC10ON | ADC10IE;
  ADC10CTL0 |= ENC+ADC10SC;
}

// restart adc10 to perfom one conversion
void retriggerADC10(){
  ADC10CTL0 |= ENC+ADC10SC;
}

// read adc10 value
static uint16_t readADC10(){
  return ADC10MEM;
}
